package org.jbjf.tasks;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.jbjf.plugin.IJBJFPluginCipher;
import org.jbjf.plugin.IJBJFPluginDatabase;
import org.jbjf.xml.JBJFDatabaseConnection;
import org.jbjf.xml.JBJFPluginDefinition;
import org.jbjf.xml.JBJFSQLDefinition;
import org.jbjf.xml.JBJFSQLParameter;

/**
 * <p>
 * The <code>LoadResultSet</code> provides...
 * 
 * org.jbjf.tasks
 * jbjf-database-tutorials
 * </p>
 * <p>
 * <h3>Details</h3>
 * <hr>
 * <h4>Input Resources</h4>
 * <table border='1' width='100%'>
 * <thead>
 *  <tr>
 *      <td width='15%'>Method</td>
 *      <td width='1%'>&nbsp;</td>
 *      <td width='15%'>Usage</td>
 *      <td width='1%'>&nbsp;</td>
 *      <td>Description/Comments</td>
 *  </tr>
 * </thead>
 *  <tr valign='top'>
 *      <td>getEndTime/setEndTime</td>
 *      <td>&nbsp;</td>
 *      <td>Runtime</td>
 *      <td>&nbsp;</td>
 *      <td>
 *      Provides a JBJF ...
 *      </td>
 *  </tr>
 * </table>
 * </p>
 * <p>
 * <pre>
 * </pre>
 * </p>
 * <p>
 * @author  Adym S. Lincoln<br>
 * Copyright (C) 2007-2013. JBJF All rights reserved.
 * @version 2.0.0
 * @since   1.0.0
 * </p>
 * <p>
 * @see org.jbjf.core.AbstractTask
 * </p>
 */

public class LoadResultSet extends AbstractSQLTask {
    /** 
     * Stores a fully qualified class name.  Used for debugging and 
     * auditing.
     * @since 1.0.0
     */
    public static final String ID = LoadResultSet.class.getName();

    /** 
     * Stores the class name, primarily used for debugging and so 
     * forth.  Used for debugging and auditing.
     * @since 1.0.0
     */
    private String SHORT_NAME = "LoadResultSet()";

    /** 
     * Stores a <code>SYSTEM IDENTITY HASHCODE</code>.  Used for
     * debugging and auditing.
     * @since 1.0.0
     */
    private String SYSTEM_IDENTITY = String.valueOf(System.identityHashCode(this));

    /**
     * Class property that stores the Job Stack key where the 
     * <code>DelimitedDocumentAdapter</code> is stored.
     */
    protected   String                      jobStackKey;

    /**
     * Class property that stores the java.sql.ResultSet from the
     * Job Stack.
     */
    protected   ResultSet                   resultSet = null;
    
    /**
     * Class property that stores a list of <code>String</code>
     * field names that will be processed.
     */
    private     ArrayList                   fields = null;

    /**
     * API constants that define the various &lt;resource&gt; 
     * keys that the Task is expecting to use.
     */
    private String REZ_KEY_SQLDEFINITION    = "sql-definition";
    private String REZ_KEY_CONNECTION       = "connection";
    private String REZ_KEY_PLUGINDATABASE   = "plugin-database";
    private String REZ_KEY_PLUGINCIPHER     = "plugin-cipher";
    private String REZ_KEY_RESULTSET        = "result-set";
    private String REZ_KEY_JOBSTACK         = "job-stack-key";

    /**
     * Default constructor.  Needed by JBJF for dynamic allocation.
     */
    public LoadResultSet() {
        // CHECK : Abstract classes need to always check to make sure
        // a parent class has NOT already initialized the required
        // resources.
        if ( getRequiredResources() == null ) { 
            mtaskRequired = new ArrayList();
        }
        getRequiredResources().add("sql-definition");   /* sql statement to run */
        getRequiredResources().add("connection");       /* database connection  */
        getRequiredResources().add("plugin-database");  /* JDBC management      */
        fields = new ArrayList();
        fields.add("id_customer");
        fields.add("first_name");
        fields.add("middle_name");
        fields.add("last_name");
        fields.add("job_title");

    }

    /* (non-Javadoc)
     * @see org.jbjf.tasks.AbstractSQLTask#runTask(java.util.HashMap)
     */
    @Override
    public void runTask(HashMap jobStack) throws Exception {
        getLog().info( SHORT_NAME + "...Start..." );
        
        /* Re-push the job stack onto the job stack to ensure that
         * no additional parameters were added since initTask() ran.
         */
        this.mjobStack = jobStack;

        if ( hasRequiredResources() ) {
            try { 
                /* RESOLVE RESOURCES : Locate and store the individual
                 * objects from the Job Stack into the class properties...
                 */
                setJobStackKey ( (String)getResources().get ( REZ_KEY_JOBSTACK ) );
                setResultSet ( 
                    (ResultSet) getParameters ().get ( getJobStackKey() 
                    )
                );
                
                /* Use the job stack key to fetch the database
                 * connection...all the plugin definition and 
                 * runtime plugin management is handled by the 
                 * AbstractSQLTask.
                 */
                if ( getConnection() == null ) {
                    throw new Exception ( 
                        "A database connection could not be established.  Please"
                        + " check all properties in the <connection> element"
                        + " named [" + getConnectionDefinition().getName () + "]"
                    );
                }
                else {
                    Statement   ldtbStatement = null;
                    // STATEMENT : Create the database statement...
                    //
                    ldtbStatement = getConnection().createStatement ();
                    
                    // LEADING ZERO : Provides a simple decimal format
                    // with leading seros that allows us to iterate
                    // thru all the JBJFSQLParamaters, p001, p002, p003, etc...
                    // 
                    DecimalFormat   leadingZeros = new DecimalFormat ( "000" );

                    int             rowCount = 0;
                    int             successCount = 0;
                    
                    // FIELDS : The following code can be used to populate
                    // the fields class property on the fly.  BUT
                    // in order to do this, the column names in the
                    // ResultSet MUST match the column names in the
                    // database table, AND in the same sequence.
                    // 
                    //ResultSetMetaData rsMetaData = getResultSet().getMetaData();
                    //int numberOfColumns = rsMetaData.getColumnCount();
                    //getLog().debug( "Loading fields...Number of Columns [" + numberOfColumns + "]" );
                    // get the column names; column indexes start from 1
                    //for (int i = 1; i < numberOfColumns + 1; i++) {
                    //    String columnName = rsMetaData.getColumnName(i);
                    //    getFields().add(columnName);
                    //}

                    // ROWS : Iterate thru all the rows in the document...
                    // Because it's a ResultSet, we use w while loop
                    // to iterate thru the rows.
                    // 
                    getLog().info ( SHORT_NAME + " Loading Results..."  );
                    
                    // DATA : Iterate thru the rows of data...
                    while ( getResultSet().next() ){
                        Iterator    lfieldIter = getFields().iterator();
                        int         colIdx = 0;
                        // FIELDS : Iterate thru the fields on a row...
                        // For each field, locate the corresponding sql-definition
                        // parameter and replace the value.
                        while ( lfieldIter.hasNext() ) {
                            colIdx++;
                            String lparmKey = "p" + leadingZeros.format(colIdx);
                            JBJFSQLParameter ltheParm = (JBJFSQLParameter) getSqlDefinition().getParameters().get(lparmKey);
                            String ltheValue = getResultSet().getString((String)lfieldIter.next());
                            if (ltheParm != null) {
                                ltheValue = scrubData(ltheValue);
//                                if ( lparmKey.equalsIgnoreCase("p001") ) {
//                                    ltheValue = scrubData(ltheValue);
//                                }
//                                if (ltheValue != null) {
//                                    ltheValue = "'" + ltheValue + "'";
//                                    ltheValue = scrubData(ltheValue);
//                                }
                                ltheParm.setValue(ltheValue);
                            }
                        }   /* END while ( fields )  */

                        // SQL : Resolve parameters and fetch the SQL...
                        //
                        String lstrSQL = getSqlDefinition().getStatement();

                        // NULL : Force all 'null' to true NULL...
                        lstrSQL = lstrSQL.replaceAll( "'null'", "null" ); 

                        // RUN : Run the SQL against the database...
                        //
                        getLog().debug(SHORT_NAME + " SQL ");
                        getLog().debug(
                            "-------------------------------------------------");
                        getLog().debug(lstrSQL);
                        getLog().debug(
                            "-------------------------------------------------");
                        try {
                            ldtbStatement.executeUpdate ( lstrSQL );
                            successCount++;
                        }
                        catch (SQLException lsqlXcp) {
                            getLog().warn( SHORT_NAME + " SQLException on Record [" + rowCount + "]" );
                            getLog().warn( lstrSQL );
                        }
                        catch ( Exception ltheXcp ) {
                            throw ltheXcp;
                        }
                        rowCount++;

                    } //end while dataSource.next()*/

                    // CHECK : If the rowcount doesn't match the
                    // success count, then some records didn't load...
                    //
                    if ( rowCount != successCount ) {
                        getLog().warn ( 
                            SHORT_NAME 
                            + " Not all rows were successfully loaded.  " 
                            + successCount + " of " 
                            + rowCount + " were loaded!" 
                        );
                    }
                    else {
                        getLog().info ( 
                            SHORT_NAME 
                            + " Rows were successfully loaded [" 
                            + successCount + "]" 
                        );
                    }

                    // GC : Check for the DelimitedDocument on the job
                    // stack and remove it for GC...
                    // 
                    String  lkeyDoc = (String)getResources().get( REZ_KEY_JOBSTACK );
                    if ( getParameters().containsKey(lkeyDoc) ) {
                        getParameters().remove(lkeyDoc);
                    }
                }

            }
            catch ( Exception ltheXcp ) {
                getLog().fatal(ltheXcp.getMessage());
                throw ltheXcp;
            }
            finally { 
                getPluginDatabaseRuntime().closeConnection ();
            }
        }
        getLog().info( SHORT_NAME + "...Complete..." );
    }

    /**
     * Utility method that cleans out special characters that could
     * cause an SQL statement to fail.
     * <p>
     * 
     * @param someData
     *            A <code>String</code> containing the data to cleanse.
     * @return Returns a <code>String</code> value minus single-quote (')
     * or line feeds (\n)
     */
    public String scrubData(String someData) throws Exception {
        String lstrResults = someData;

        if ( lstrResults.indexOf('\'') > -1 ) {
            lstrResults = lstrResults.replace('\'', ' ');
        }
        if ( lstrResults.indexOf('\n') > -1 ) {
            lstrResults = lstrResults.replace('\n', ' ');
        }
        if ( lstrResults.indexOf(',') > -1 ) {
            lstrResults = lstrResults.replace(",","");
        }
        return lstrResults.trim();
    }

    public String getJobStackKey() {
        return jobStackKey;
    }

    public void setJobStackKey(String jobStackKey) {
        this.jobStackKey = jobStackKey;
    }

    public ResultSet getResultSet() {
        return resultSet;
    }

    public void setResultSet(ResultSet resultSet) {
        this.resultSet = resultSet;
    }

    public ArrayList getFields() {
        return fields;
    }

    public void setFields(ArrayList fields) {
        this.fields = fields;
    }

}
